@@ -2,6 +2,7 @@ from typing import List, Union
 import copy
 import numpy as np
 from ..rule import RegressionRule, ClassificationRule
+from ..ruleset import RuleSet
 from sklearn.tree import DecisionTreeClassifier, DecisionTreeRegressor
 from ..condition import HyperrectangleCondition
 
@@ -17,8 +18,11 @@ def extract_rules_from_tree(
     xmins: Union[List[float], np.ndarray],
     xmaxs: Union[List[float], np.ndarray],
     features_names: List[str] = None,
+    classes_names: List[str] = None,
     get_leaf: bool = False,
-) -> Union[List[RegressionRule], List[ClassificationRule]]:
+    remember_activation: bool = True,
+    stack_activation: bool = False,
+) -> RuleSet:
     """To extract rules from a sklearn decision tree
 
     features are the list of X names.
@@ -65,18 +69,23 @@ def extract_rules_from_tree(
     Parameters
     ----------
     tree : Union[sklearn.tree.DecisionTreeRegressor, sklearn.tree.DecisionTreeClassifier]
-
     features_names: List[str],
         the list of X names
-
+    classes_names: List[str],
+        the list of Y classes names
     xmins: Union[List[int], List[float], np.ndarray]
         min values of each xs, one entry per x
-
     xmaxs: Union[List[int], List[float], np.ndarray]
         max values of each xs, one entry per x
-
     get_leaf: Boolean type
         To return only the leaf of the tree.
+    remember_activation: bool
+        The activation of the RuleSet is the logical OR of the activation of all its rules. It is only computed
+        if remember_activation is True. (default value = True)
+    stack_activation: bool
+        If True, the RuleSet will keep in memory 2-D np.ndarray containing the activations of all its rules. This
+        can take a lot of memory, but can save time if you apply numpy methods on this stacked vector instead of on
+        each rule separately. (default value = False)
 
     Returns
     -------
@@ -90,7 +99,7 @@ def extract_rules_from_tree(
 
     def visitor(node, depth, condition=None, rules_list=None):
         if rules_list is None:
-            rules_list = []
+            rules_list = RuleSet(remember_activation=remember_activation, stack_activation=stack_activation)
         if decision_tree.feature[node] != _tree.TREE_UNDEFINED:
             # If
             new_condition = HyperrectangleCondition(
@@ -121,8 +130,15 @@ def extract_rules_from_tree(
             if get_leaf is False:
                 if isinstance(tree, DecisionTreeClassifier):
                     new_rule = ClassificationRule(copy.deepcopy(new_condition))
+                    if classes_names is None:
+                        new_rule._prediction = [pred for pred in enumerate(decision_tree.value[node][0])]
+                    else:
+                        new_rule._prediction = [
+                            (classes_names[i], pred) for i, pred in enumerate(decision_tree.value[node][0])
+                        ]
                 else:
                     new_rule = RegressionRule(copy.deepcopy(new_condition))
+                    new_rule._prediction = decision_tree.value[node][0][0]
                 rules_list.append(new_rule)
 
             # Execute the current function on the left of the current node
@@ -166,17 +182,33 @@ def extract_rules_from_tree(
             if get_leaf is False:
                 if isinstance(tree, DecisionTreeClassifier):
                     new_rule = ClassificationRule(copy.deepcopy(new_condition))
+                    if classes_names is None:
+                        new_rule._prediction = [pred for pred in enumerate(decision_tree.value[node][0])]
+                    else:
+                        new_rule._prediction = [
+                            (classes_names[i], pred) for i, pred in enumerate(decision_tree.value[node][0])
+                        ]
                 else:
                     new_rule = RegressionRule(copy.deepcopy(new_condition))
+                    new_rule._prediction = decision_tree.value[node][0][0]
                 rules_list.append(new_rule)
 
             rules_list = visitor(decision_tree.children_right[node], depth + 1, new_condition, rules_list)
 
         elif get_leaf:
             if isinstance(tree, DecisionTreeClassifier):
-                rules_list.append(ClassificationRule(copy.deepcopy(condition)))
+                new_rule = ClassificationRule(copy.deepcopy(condition))
+                if classes_names is None:
+                    new_rule._prediction = [pred for pred in enumerate(decision_tree.value[node][0])]
+                else:
+                    new_rule._prediction = [
+                        (classes_names[i], pred) for i, pred in enumerate(decision_tree.value[node][0])
+                    ]
+                rules_list.append(new_rule)
             else:
-                rules_list.append(RegressionRule(copy.deepcopy(condition)))
+                new_rule = RegressionRule(copy.deepcopy(condition))
+                new_rule._prediction = decision_tree.value[node][0][0]
+                rules_list.append(new_rule)
 
         return rules_list
 
